﻿/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.IO;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DaxEditorSample.Test
{
    public static class WindiffAssert
    {
        private static string _tempLogFolder = null;
        private static int _counter = 0;

        /// <summary>
        ///  Normalizes expected and actual string by removing whitespaces in the beggining of lines for all element before comparison
        /// </summary>
        /// <param name="expected">Expected XMLA</param>
        /// <param name="actual">Actual XMLA</param>
        public static void AreEqualNormalizedXmla(string expected, string actual)
        {
            string normalizedExpected = NormalizeWhitespacesInText(expected);
            string normalizedActual = NormalizeWhitespacesInText(expected);

            if (!string.Equals(normalizedExpected, normalizedActual))
            {
                if(_tempLogFolder == null)
                {
                    _tempLogFolder = Path.Combine(Path.GetTempPath(), Guid.NewGuid().ToString());
                    Directory.CreateDirectory(_tempLogFolder);
                }

                string expectedFileName = Path.Combine(_tempLogFolder, _counter.ToString("00") + "_expected.txt");
                File.WriteAllText(expectedFileName, expected);
                string actualFileName = Path.Combine(_tempLogFolder, _counter.ToString("00") + "_actual.txt");
                File.WriteAllText(actualFileName, actual);

                string expectedNormalizedFileName = Path.Combine(_tempLogFolder, _counter.ToString("00") + "_expectedNormalized.txt");
                File.WriteAllText(expectedFileName, normalizedExpected);
                string actualNormalizedFileName = Path.Combine(_tempLogFolder, _counter.ToString("00") + "_actualNormalized.txt");
                File.WriteAllText(actualFileName, normalizedActual);

                string winDiffCommandLine = string.Format(@"Windiff.exe ""{0}"" ""{1}""", expectedFileName, actualFileName);
                string winDiffCommandLineForNormalized = string.Format(@"Windiff.exe ""{0}"" ""{1}""", expectedNormalizedFileName, actualNormalizedFileName);
                Assert.Fail("Different XMLA.  Normalized diff: \r\n" + winDiffCommandLineForNormalized);
                Assert.Fail("Different XMLA.  Actual diff: \r\n" + winDiffCommandLine);
            }
        }

        /// <summary>
        /// Normalizes string by removing whitespaces from beggining of every line
        /// </summary>
        /// <param name="input">Input string</param>
        /// <returns>Normalized string</returns>
        private static string NormalizeWhitespacesInText(string input)
        {
            if (null == input)
                return input;

            var workString = input.Replace("\r\n", "\n");
            workString = input.Replace("\r", "\n");
            var lines = input.Split('\n');
            var sbResult = new StringBuilder();
            foreach (var line in lines)
            {
                sbResult.AppendLine(line.TrimStart(new char[] { ' ', '\t' }));
            }

            return sbResult.ToString();
        }
    }
}
