﻿/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.Collections.Generic;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.TextManager.Interop;
using Microsoft.VisualStudio.Package;
using Microsoft.DaxEditor.GeneratorSource;
using Babel.Parser;
using Microsoft.DaxEditor;

namespace Babel
{
    public class AuthoringScope : Microsoft.VisualStudio.Package.AuthoringScope
    {
        public AuthoringScope(Source source)
        {
            this._source = source;
        }

        public AuthoringScope(object parseResult, Source source)
        {
            this._parseResult = parseResult;
            this._source = source;

            // how should this be set?
            this._resolver = new Resolver(this._source);
        }

        Source _source;
        object _parseResult;
        IASTResolver _resolver;

        // ParseReason.QuickInfo
        public override string GetDataTipText(int line, int col, out TextSpan span)
        {
            TokenInfo tokenInfo = this._source.GetTokenInfo(line, col);

            span = new TextSpan();
            span.iStartLine = line;
            span.iEndLine = line;
            span.iStartIndex = tokenInfo.StartIndex;
            span.iEndIndex = tokenInfo.EndIndex + 1;

            if(TokenType.Identifier == tokenInfo.Type)
            {
                // TODO: Hack, to be re-writen when tree with Declaration is ready
                foreach (var function in new DaxFunctions())
                {
                    if(string.Equals(function.Name, this._source.GetText(span), StringComparison.InvariantCultureIgnoreCase))
                    {
                        return function.Description;
                    }
                }
            }


            return null;
        }

        // ParseReason.CompleteWord
        // ParseReason.DisplayMemberList
        // ParseReason.MemberSelect
        // ParseReason.MemberSelectAndHilightBraces
        public override Microsoft.VisualStudio.Package.Declarations GetDeclarations(IVsTextView view, int line, int col, TokenInfo info, ParseReason reason)
        {
            IList<Declaration> declarations = null;

            int lastHResult;
            TokenInfo parentTokenInfo;
            switch (reason)
            {
                case ParseReason.CompleteWord:
                    CompletionContext completionContext = new CompletionContext();
                    completionContext.TokenToComplete = info.Token;
                    completionContext.ShouldAppendClosingOpeningBrace = false;
                    completionContext.ParentTokenName = string.Empty;

                    string tokenBeggining, parentTokenName;
                    lastHResult = view.GetTextStream(line, info.StartIndex, line, info.EndIndex + 1, out tokenBeggining);
                    if (lastHResult != VSConstants.S_OK)
                        return null; //

                    completionContext.TokenBeggining = tokenBeggining;

                    if (info.Token == (int)Tokens.PARTIALCOLUMNNAME || info.Token == (int)Tokens.COLUMNNAME)
                    { // Need a parent for a column
                        parentTokenInfo = _source.GetTokenInfo(line, info.StartIndex - 1);
                        if (parentTokenInfo.Token == (int)Tokens.TABLENAME)
                        {
                            // Table name is leading
                            lastHResult = view.GetTextStream(line, parentTokenInfo.StartIndex, line, parentTokenInfo.EndIndex + 1, out parentTokenName);
                            if (lastHResult == VSConstants.S_OK)
                            {
                                completionContext.ParentTokenName = parentTokenName;
                            }
                        }
                    }

                    declarations = _resolver.FindCompletions(completionContext, line, col);
                    break;

                case ParseReason.DisplayMemberList:
                case ParseReason.MemberSelect:
                case ParseReason.MemberSelectAndHighlightBraces:
                    // Obtain the parent token info before 
                    parentTokenInfo = _source.GetTokenInfo(line, info.StartIndex - 1);

                    if (parentTokenInfo.Token == (int)Tokens.TABLENAME || parentTokenInfo.Token == (int)Tokens.ESCAPEDTABLENAME)
                    {
                        // Table name is leading
                        string parentObjectName = string.Empty;
                        lastHResult = view.GetTextStream(line, parentTokenInfo.StartIndex, line, parentTokenInfo.EndIndex + 1, out parentObjectName);
                        if (lastHResult == VSConstants.S_OK)
                        {
                            declarations = _resolver.FindMembers(parentObjectName.Trim('\''), line, col);
                        }
                    }
                    else if (parentTokenInfo.Type != TokenType.Text && parentTokenInfo.Type != TokenType.String)
                    {
                        // No table name follows the square bracket
                        declarations = _resolver.FindMembers(string.Empty, line, col);
                    }

                    break;

                default:
                    throw new ArgumentException("reason");
            }

            return new Declarations(declarations);
        }

        // ParseReason.GetMethods
        public override Microsoft.VisualStudio.Package.Methods GetMethods(int line, int col, string name)
        {
            return new Methods(_resolver.FindMethods(_parseResult, line, col, name));
        }

        // ParseReason.Goto
        public override string Goto(VSConstants.VSStd97CmdID cmd, IVsTextView textView, int line, int col, out TextSpan span)
        {
            // throw new System.NotImplementedException();
            span = new TextSpan();
            return null;
        }
    }
}