﻿/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.

THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Package;
using System.ComponentModel;
using Microsoft.Win32;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace Babel
{
    public class DaxDocumentProperties : DocumentProperties
    {
        public DaxDocumentProperties(CodeWindowManager mgr)
            : base(mgr)
        {
        }

        private string _connectionString;

        [DisplayNameAttribute("Connection string")]
        [CategoryAttribute("General")]
        [DescriptionAttribute("Connection string to SSAS database")]
        [ReadOnly(true)]
        public string ConnectionString
        {
            get
            {
                if (string.IsNullOrEmpty(_connectionString))
                {
                    _connectionString = Application.UserAppDataRegistry.GetValue("ConnectionString") as string;
                }
                return _connectionString;
            }
            set
            {
                _connectionString = value;
                if(!string.IsNullOrEmpty(_connectionString))
                    Application.UserAppDataRegistry.SetValue("ConnectionString", _connectionString);
            }
        }

        internal string DatabaseName
        {
            get
            {
                var match = Regex.Match(ConnectionString, @"Catalog\s*=([^;]*);*");
                if (match.Success)
                {
                    Debug.Assert(match.Groups.Count > 1);
                    return match.Groups[1].Value;
                }
                else
                {
                    Debug.Assert(false, "No Catalog in the connection string");
                    return string.Empty;
                }
            }
        }
    }
}
