﻿/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

namespace Microsoft.DaxEditor
{
    public class DaxMeasure
    {
        /// <summary>
        /// Name of the table where measure belongs to.  TableName should be escaped if it is required.
        /// </summary>
        public string TableName { get; set; }


        private string _name;
        /// <summary>
        /// Measure name, without brackets
        /// </summary>
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                System.Diagnostics.Debug.Assert(value != null && !value.StartsWith("[") && !value.EndsWith("]"));
                _name = value;
            }
        }

        /// <summary>
        /// Measure expression - text that goes after = in the measure definition.
        /// </summary>
        public string Expression { get; set; }

        private string _fullText;
        /// <summary>
        /// Full text of a measure, starting with CREATE MEASURE. Does not include ';'.
        /// </summary>
        public string FullText
        {
            get
            {
                return _fullText;
            }
            set
            {
                if (value != null)
                {
                    // Normalize XML EOF '\n' to Windows EOL '\r\n'
                    value = value.Replace("\r\n", "\n");
                    value = value.Replace("\n", "\r\n");
                }
                _fullText = value;
            }
        }
        public string NameInBrackets
        {
            get
            {
                return "[" + Name + "]";
            }
        }

        /// <summary>
        /// Calculation property for the measure
        /// </summary>
        public DaxCalcProperty CalcProperty { get; set; }
    }
}
