﻿#/***************************************************************************
#
#Copyright (c) Microsoft Corporation. All rights reserved.
#
#THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
#ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
#IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
#PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
#
#***************************************************************************/

# Generates code with DAX keywords

[array]$keyWords = @()
$keyWords += New-Object object | add-member NoteProperty Name "evaluate" -passThru  | add-member NoteProperty Description "Evaluate literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "define" -passThru  | add-member NoteProperty Description "Define literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "measure" -passThru  | add-member NoteProperty Description "Measure literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Order" -passThru  | add-member NoteProperty Description "Order literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "by" -passThru  | add-member NoteProperty Description "BY literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "TRUE" -passThru  | add-member NoteProperty Description "True constant" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "FALSE" -passThru  | add-member NoteProperty Description "False constant" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "ASC" -passThru  | add-member NoteProperty Description "Ascending sort order" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "DESC" -passThru  | add-member NoteProperty Description "Descending sort order" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "DAY" -passThru  | add-member NoteProperty Description "Period equal to 1 day in time intelligence functions" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "MONTH" -passThru  | add-member NoteProperty Description "Period equal to 1 month in time intelligence functions" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "YEAR" -passThru  | add-member NoteProperty Description "Period equal to 1 year in time intelligence functions" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Create" -passThru  | add-member NoteProperty Description "CREATE literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "CALCULATE" -passThru  | add-member NoteProperty Description "CALCULATE keyword" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "CALCULATION" -passThru  | add-member NoteProperty Description "CALCULATION literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "PROPERTY" -passThru  | add-member NoteProperty Description "PROPERTY literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "General" -passThru  | add-member NoteProperty Description "General calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "NumberDecimal" -passThru  | add-member NoteProperty Description "NumberDecimal calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "NumberWhole" -passThru  | add-member NoteProperty Description "NumberWhole calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Percentage" -passThru  | add-member NoteProperty Description "Percentage calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Scientific" -passThru  | add-member NoteProperty Description "Scientific calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Currency" -passThru  | add-member NoteProperty Description "Currency calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "DateTimeCustom" -passThru  | add-member NoteProperty Description "DateTimeCustom calculation property format type" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Accuracy" -passThru  | add-member NoteProperty Description "Accuracy literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "ThousandSeparator" -passThru  | add-member NoteProperty Description "ThousandSeparator literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "Format" -passThru  | add-member NoteProperty Description "Format literal" -passThru  
$keyWords += New-Object object | add-member NoteProperty Name "AdditionalInfo" -passThru  | add-member NoteProperty Description "AdditionalInfo literal" -passThru  

# DaxKeywords_Gen.cs
$pathToGeneratedFile = (Get-Location).ToString() + "\DaxKeywords_Gen.cs"
if(![System.IO.File]::Exists($pathToGeneratedFile)) {
   throw "$pathToGeneratedFile does not exist.  The script needs to run from the location of the file."
}
$generatedCode = ""
$keyWords | %{
   $generatedCode += [string]::Format("
      this.Add(new DaxKeywordDescription() {{ Name = @""{0}"", Description = @""{1}""}});",
      $_.Name.ToUpperInvariant(),
      $_.Description
      )
}

$tokenBeginReplace = "/* Begin generated code */"
$tokenEndReplace = "/* End generated code */"
$inputText = Get-Content $pathToGeneratedFile | Out-String
[string]$resultText = $inputText.Substring(0, $inputText.IndexOf($tokenBeginReplace) + $tokenBeginReplace.Length)
$resultText += $generatedCode;
$resultText += $inputText.Substring($inputText.IndexOf($tokenEndReplace) - 2);
$resultText | Out-File -FilePath $pathToGeneratedFile -Encoding UTF8


# lexer.lex
$pathToGeneratedFile = (Get-Location).ToString() + "\lexer.lex"
if(![System.IO.File]::Exists($pathToGeneratedFile)) {
   throw "$pathToGeneratedFile does not exist.  The script needs to run from the location of the file."
}

$fieldSize = (($keyWords | Sort-Object -property @{Expression={$_.Name.Length}; Ascending=$false} | Select-Object -First 1).Name.Length + 2) * 4

$generatedCode = ""
$listOfTokensString = "%token"

$keyWords | %{
$currentKeyword = $_.Name.ToUpperInvariant()
$listOfTokensString += " KW$currentKeyword"
$currentKeyword.ToCharArray() | %{ $generatedCode += "[" + $_ + [char]::ToLowerInvariant($_) + "]"}
$spacesNeeded = $fieldSize - $currentKeyword.Length * 4;
$generatedCode += [string]::Format("{0,$spacesNeeded}", " ");
$generatedCode += "{ return (int)Tokens.KW$currentKeyword; }" + [Environment]::NewLine;
}
$inputText = Get-Content $pathToGeneratedFile | Out-String
[string]$resultText = $inputText.Substring(0, $inputText.IndexOf($tokenBeginReplace) + $tokenBeginReplace.Length + 2)
$resultText += $generatedCode;
$resultText += $inputText.Substring($inputText.IndexOf($tokenEndReplace));
$resultText | Out-File -FilePath $pathToGeneratedFile -Encoding ASCII

# parser.y
$pathToGeneratedFile = (Get-Location).ToString() + "\parser.y"
$inputText = Get-Content $pathToGeneratedFile | Out-String
$tokenBeginReplace = "/* Begin generated list of tokens */"
$tokenEndReplace = "/* End generated list of tokens */"
[string]$resultText = $inputText.Substring(0, $inputText.IndexOf($tokenBeginReplace) + $tokenBeginReplace.Length + 2)
$resultText += $listOfTokensString;
$resultText += $inputText.Substring($inputText.IndexOf($tokenEndReplace) - 2);
$resultText | Out-File -FilePath $pathToGeneratedFile -Encoding ASCII
