/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

%using Babel;
%using Babel.Parser;
%using Microsoft.DaxEditor.GeneratorSource;
%using System.Diagnostics;

%namespace Babel.Lexer


%x COMMENT

%{
       int GetIdToken(string txt)
       {
            if(DaxFunctions.GetFunctionsTree().Contains(txt))
            {
                return (int)Tokens.FUNCTION;
            }

            return (int)Tokens.TABLENAME;
       }
       
       internal void LoadYylval()
       {
           yylval.str = tokTxt;
           yylloc = new LexLocation(tokLin, tokCol, tokLin, tokECol);
       }
       
       public override void yyerror(string s, params object[] a)
       {
           if (handler != null) handler.AddError(s, tokLin, tokCol, tokECol - tokCol + 1, 2);  // Sev 2 = error
       }

       public void SetSourceText(string text)
       {
            SetSource(text, 0);
            InitializeLineStartArray(text);
       }

       private int[] _lineStartArray = null;

       public override string GetText(LexLocation location)
       {
            Debug.Assert(_lineStartArray != null);

            int startPosition = FindBegginingOfLinePosition(location.sLin) + location.sCol;
            int endPosition = FindBegginingOfLinePosition(location.eLin) + location.eCol;

            Debug.Assert(endPosition > startPosition);

            return buffer.GetString(startPosition, endPosition);
       }

       private void InitializeLineStartArray(string text)
       {
            Debug.Assert(_lineStartArray == null);
            var lineStartList = new List<int>();
            lineStartList.Add(0);

            // Expected EOL - \n and \r\n
            var textChars = text.ToCharArray();
            for(int position = 1; position < textChars.Length; ++position)
            {
                char prevChar = textChars[position - 1];
                if(prevChar == '\n')
                    lineStartList.Add(position);
            }

            _lineStartArray = lineStartList.ToArray();
       }

       /// <summary>
       /// Returns start position for the specified line.  Line is 1-based index.
       /// </summary>
       private int FindBegginingOfLinePosition(int line)
       {
            Debug.Assert(line > 0 && line < _lineStartArray.Length + 1);
            return _lineStartArray[line - 1];
       }
%}


White0          [ \t\r\f\v]
White           {White0}|\n

CmntStart            \/\*
CmntEnd              \*\/
LineComment1         --
LineComment2         \/\/
ABStar               [^\*\n]*
StringBeginEnd       \"
StringContent        [^\"\n]*
ColumnName           [^\]\n]+
TableNameEscaped     [^\'\n]+
TableNameNotEscaped  [a-zA-Z_][a-zA-Z0-9_]

%%
/* Begin generated code */
[Ee][Vv][Aa][Ll][Uu][Aa][Tt][Ee]                                            { return (int)Tokens.KWEVALUATE; }
[Dd][Ee][Ff][Ii][Nn][Ee]                                                    { return (int)Tokens.KWDEFINE; }
[Mm][Ee][Aa][Ss][Uu][Rr][Ee]                                                { return (int)Tokens.KWMEASURE; }
[Oo][Rr][Dd][Ee][Rr]                                                        { return (int)Tokens.KWORDER; }
[Bb][Yy]                                                                    { return (int)Tokens.KWBY; }
[Tt][Rr][Uu][Ee]                                                            { return (int)Tokens.KWTRUE; }
[Ff][Aa][Ll][Ss][Ee]                                                        { return (int)Tokens.KWFALSE; }
[Aa][Ss][Cc]                                                                { return (int)Tokens.KWASC; }
[Dd][Ee][Ss][Cc]                                                            { return (int)Tokens.KWDESC; }
[Dd][Aa][Yy]                                                                { return (int)Tokens.KWDAY; }
[Mm][Oo][Nn][Tt][Hh]                                                        { return (int)Tokens.KWMONTH; }
[Yy][Ee][Aa][Rr]                                                            { return (int)Tokens.KWYEAR; }
[Cc][Rr][Ee][Aa][Tt][Ee]                                                    { return (int)Tokens.KWCREATE; }
[Cc][Aa][Ll][Cc][Uu][Ll][Aa][Tt][Ee]                                        { return (int)Tokens.KWCALCULATE; }
[Cc][Aa][Ll][Cc][Uu][Ll][Aa][Tt][Ii][Oo][Nn]                                { return (int)Tokens.KWCALCULATION; }
[Pp][Rr][Oo][Pp][Ee][Rr][Tt][Yy]                                            { return (int)Tokens.KWPROPERTY; }
[Gg][Ee][Nn][Ee][Rr][Aa][Ll]                                                { return (int)Tokens.KWGENERAL; }
[Nn][Uu][Mm][Bb][Ee][Rr][Dd][Ee][Cc][Ii][Mm][Aa][Ll]                        { return (int)Tokens.KWNUMBERDECIMAL; }
[Nn][Uu][Mm][Bb][Ee][Rr][Ww][Hh][Oo][Ll][Ee]                                { return (int)Tokens.KWNUMBERWHOLE; }
[Pp][Ee][Rr][Cc][Ee][Nn][Tt][Aa][Gg][Ee]                                    { return (int)Tokens.KWPERCENTAGE; }
[Ss][Cc][Ii][Ee][Nn][Tt][Ii][Ff][Ii][Cc]                                    { return (int)Tokens.KWSCIENTIFIC; }
[Cc][Uu][Rr][Rr][Ee][Nn][Cc][Yy]                                            { return (int)Tokens.KWCURRENCY; }
[Dd][Aa][Tt][Ee][Tt][Ii][Mm][Ee][Cc][Uu][Ss][Tt][Oo][Mm]                    { return (int)Tokens.KWDATETIMECUSTOM; }
[Aa][Cc][Cc][Uu][Rr][Aa][Cc][Yy]                                            { return (int)Tokens.KWACCURACY; }
[Tt][Hh][Oo][Uu][Ss][Aa][Nn][Dd][Ss][Ee][Pp][Aa][Rr][Aa][Tt][Oo][Rr]        { return (int)Tokens.KWTHOUSANDSEPARATOR; }
[Ff][Oo][Rr][Mm][Aa][Tt]                                                    { return (int)Tokens.KWFORMAT; }
[Aa][Dd][Dd][Ii][Tt][Ii][Oo][Nn][Aa][Ll][Ii][Nn][Ff][Oo]                    { return (int)Tokens.KWADDITIONALINFO; }
/* End generated code */

[Mm][Ee][Mm][Bb][Ee][Rr][^;]*           { return (int)Tokens.MDXCODE; }
[Kk][Pp][Ii][^;]*                       { return (int)Tokens.MDXCODE; }
[Al][Ll][Tt][Ee][Rr][^;]*               { return (int)Tokens.KWALTER; }

\[{ColumnName}                          { return (int)Tokens.PARTIALCOLUMNNAME; }
'{TableNameEscaped}                     { return (int)Tokens.PARTIALTABLENAME; }
\[{ColumnName}\]                        { return (int)Tokens.COLUMNNAME; }
'{TableNameEscaped}'                    { return (int)Tokens.ESCAPEDTABLENAME; }
{TableNameNotEscaped}                   { return GetIdToken(yytext); }

[a-zA-Z_][a-zA-Z0-9_]*                  { return GetIdToken(yytext); }
[0-9]+(\.[0-9]+([eE]([\+\-])*[0-9]+)*)*                       { return (int)Tokens.NUMBER; }

;                         { return (int)';';    }
,                         { return (int)',';    }
\(                        { return (int)'(';    }
\)                        { return (int)')';    }
\{                        { return (int)'{';    }
\}                        { return (int)'}';    }
\^                        { return (int) Tokens.POW;    }
\+                        { return (int)'+';    }
\-                        { return (int)'-';    }
\*                        { return (int)'*';    }
\/                        { return (int)'/';    }
\!                        { return (int)'!';    }
=                         { return (int)Tokens.EQ;  }
\<\>                      { return (int)Tokens.NEQ;   }
\>                        { return (int)Tokens.GT; }
\>=                       { return (int)Tokens.GTE;    }
\<                        { return (int)Tokens.LT;     }
\<=                       { return (int)Tokens.LTE;    }
\&                        { return (int)'&';    }
\&\&                      { return (int)Tokens.AMPAMP; }
\|                        { return (int)'|';    }
\|\|                      { return (int)Tokens.BARBAR; }
\.                        { return (int)'.';    }
\[                        { return (int) Tokens.LEFTSQUAREBRACKET;	}
\]                        { return (int) Tokens.RIGHTSQUAREBRACKET;    }

{StringBeginEnd}{StringContent}{StringBeginEnd}         { return (int)Tokens.STRING; }

{CmntStart}{ABStar}\**{CmntEnd} { return (int)Tokens.LEX_COMMENT; }
{CmntStart}{ABStar}\**          { BEGIN(COMMENT); return (int)Tokens.LEX_COMMENT; }
<COMMENT>\n                     |                                
<COMMENT>{ABStar}\**            { return (int)Tokens.LEX_COMMENT; }
<COMMENT>{ABStar}\**{CmntEnd}   { BEGIN(INITIAL); return (int)Tokens.LEX_COMMENT; }
\**{CmntEnd}                    { return (int)Tokens.LEX_COMMENT; }

{LineComment1}{ABStar}\n { return (int)Tokens.LEX_COMMENT; }
{LineComment2}{ABStar}\n { return (int)Tokens.LEX_COMMENT; }

{White0}+                  { return (int)Tokens.LEX_WHITE; }
\n                         { return (int)Tokens.LEX_WHITE; }
.                          { yyerror("illegal char");
                             return (int)Tokens.LEX_ERROR; }

%{
                      LoadYylval();
%}

%%

/* .... */















