/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

%using Microsoft.VisualStudio.TextManager.Interop
%namespace Babel.Parser
%valuetype LexValue
%partial

/* %expect 5 */


%union {
    public string str;
}


%{
    ErrorHandler handler = null;
    public void SetHandler(ErrorHandler hdlr) { handler = hdlr; }
    internal void CallHdlr(string msg, LexLocation val)
    {
        handler.AddError(msg, val.sLin, val.sCol, val.eCol - val.sCol);
    }
    internal TextSpan MkTSpan(LexLocation s) { return TextSpan(s.sLin, s.sCol, s.eLin, s.eCol); }

    internal void Match(LexLocation lh, LexLocation rh)
    {
        DefineMatch(MkTSpan(lh), MkTSpan(rh));
    }

	internal void StartFunction(LexLocation s, string token)
	{
		System.Diagnostics.Debug.WriteLine("StartName:" + token);
		Sink.StartName(MkTSpan(s), token); 
	}

	internal void StartParameters(LexLocation s)
	{
		System.Diagnostics.Debug.WriteLine("StartParameters");
		Sink.StartParameters(MkTSpan(s)); 
	}

	internal void EndParameters(LexLocation s)
	{
		System.Diagnostics.Debug.WriteLine("EndParameters");
		Sink.EndParameters(MkTSpan(s));
	}

	internal void NextParameter(LexLocation s)
	{
		System.Diagnostics.Debug.WriteLine("Next Parameter");
		Sink.NextParameter(MkTSpan(s));
	}
%}

%token FUNCTION NUMBER STRING COLUMNNAME TABLENAME ESCAPEDTABLENAME PARTIALCOLUMNNAME PARTIALTABLENAME MDXCODE KWALTER
/* Begin generated list of tokens */
%token KWEVALUATE KWDEFINE KWMEASURE KWORDER KWBY KWTRUE KWFALSE KWASC KWDESC KWDAY KWMONTH KWYEAR KWCREATE KWCALCULATE KWCALCULATION KWPROPERTY KWGENERAL KWNUMBERDECIMAL KWNUMBERWHOLE KWPERCENTAGE KWSCIENTIFIC KWCURRENCY KWDATETIMECUSTOM KWACCURACY KWTHOUSANDSEPARATOR KWFORMAT KWADDITIONALINFO
/* End generated list of tokens */

%token EQ NEQ GT GTE LT LTE POW AMPAMP BARBAR LEFTSQUAREBRACKET RIGHTSQUAREBRACKET
%token maxParseToken 
%token LEX_WHITE LEX_COMMENT LEX_ERROR



%%

Start
    : KWDEFINE Definitions KWEVALUATE Expression OrderBy
    | KWDEFINE Definitions KWEVALUATE Expression
    | KWEVALUATE Expression OrderBy
    | KWEVALUATE Expression
    | EQ Expression
    | DaxScript
    | Empty
    ;

DaxScript
    : CreateMeasure
    | CreateMeasure ';'
    | CreateMeasure ';' DaxScript
    | CreateMember
    | CreateMember ';'
    | CreateMember ';' DaxScript
    | Calculate
    | Calculate ';'
    | Calculate ';' DaxScript
    | Alter
    | Alter ';'
    | Alter ';' DaxScript
    ;

Empty
    : EOF
    ;

CubeName
    : TABLENAME
    | COLUMNNAME
    ;

CreateMeasure
    : KWCREATE KWMEASURE CubeName '.' MeasureName EQ MeasureExpression       { SpecifyFullMeasureText(@1, @7); }
    | KWCREATE KWMEASURE CubeName '.' MeasureName EQ MeasureExpression CalculationProperty
                                                                             { SpecifyFullMeasureText(@1, @7); }
    | KWCREATE KWMEASURE error '.' MeasureName EQ MeasureExpression          { CallHdlr("Cube name expected before '.'", @3); }
    | KWCREATE KWMEASURE MeasureName EQ MeasureExpression                    { SpecifyFullMeasureText(@1, @5); }
    | KWCREATE KWMEASURE MeasureName EQ MeasureExpression  CalculationProperty
                                                                             { SpecifyFullMeasureText(@1, @5); }
    | KWCREATE KWMEASURE error EQ MeasureExpression                          { CallHdlr("Measure name expected", @3); }
    ;

CreateMember
    : KWCREATE MDXCODE
    ;

Calculate
    : KWCALCULATE
    ;

Alter
    : KWALTER
    ;

CalculationPropertyFormatType
    : KWGENERAL                                                                         { SpecifyCalculationProperty(@1); }
    | KWNUMBERDECIMAL                                                                   { SpecifyCalculationProperty(@1); }
    | KWNUMBERWHOLE                                                                     { SpecifyCalculationProperty(@1); }
    | KWPERCENTAGE                                                                      { SpecifyCalculationProperty(@1); }
    | KWSCIENTIFIC                                                                      { SpecifyCalculationProperty(@1); }
    | KWCURRENCY                                                                        { SpecifyCalculationProperty(@1); }
    | KWDATETIMECUSTOM                                                                  { SpecifyCalculationProperty(@1); }
    ;

CalculationPropertyAccuracy
    : KWACCURACY EQ NUMBER                                                              { SpecifyCalcPropAccuracy(@3); }
    | KWACCURACY error                                                                  { CallHdlr("'=' is not specified", @2); }
    | KWACCURACY EQ error                                                               { CallHdlr("Value of Accuracy is not a number", @3); }
    ;

CalculationPropertyThousandSeparator
    : KWTHOUSANDSEPARATOR EQ  KWTRUE                                                    { SpecifyCalcPropThousandSeparator(true); }
    | KWTHOUSANDSEPARATOR EQ  KWFALSE                                                   { SpecifyCalcPropThousandSeparator(false); }
    | KWTHOUSANDSEPARATOR error                                                         { CallHdlr("'=' is not specified", @2); }
    | KWTHOUSANDSEPARATOR EQ error                                                      { CallHdlr("ThousandSeparator can be either TRUE or FALSE", @3); }
    ;

CalculationPropertyFormat
    : KWFORMAT EQ ESCAPEDTABLENAME                                                      { SpecifyCalcPropFormat(@3); }
    ;

CalculationPropertyAdditionalInfo
    : KWADDITIONALINFO EQ ESCAPEDTABLENAME                                              { SpecifyCalcPropAdditionalInfo(@3); }
    ;

CalculationPropertyParams
    : CalculationPropertyAccuracy
    | CalculationPropertyAccuracy CalculationPropertyParams
    | CalculationPropertyThousandSeparator
    | CalculationPropertyThousandSeparator CalculationPropertyParams
    | CalculationPropertyFormat
    | CalculationPropertyFormat CalculationPropertyParams
    | CalculationPropertyAdditionalInfo
    | CalculationPropertyAdditionalInfo CalculationPropertyParams
    ;
CalculationProperty
    : KWCALCULATION KWPROPERTY CalculationPropertyFormatType
    | KWCALCULATION KWPROPERTY error                                                    { CallHdlr("Wrong caclulation property type.  Expected types: General, NumberDecimal, NumberWhole, Percentage, Scientific, Currency, DateTimeCustom", @2); }
    | KWCALCULATION KWPROPERTY CalculationPropertyFormatType CalculationPropertyParams
    ;

Definitions
    : KWMEASURE QueryMeasureName EQ QueryMeasureExpression
    | KWMEASURE QueryMeasureName EQ QueryMeasureExpression Definitions
    ;

OrderBy
    : KWORDER KWBY OrderByList
    ;

OrderByList
    : Expression Order ',' OrderByList
    | Expression ',' OrderByList
    | Expression Order
    | Expression
    ;

Order
    : KWASC
    | KWDESC
    ;

Params1
    : Params1 ParamSeparator Expression
    | Params1 ParamSeparator
    | Expression
    ;

ParamSeparator
	: ','							{ NextParameter(@1); }
	;

ParenthesisParameters
    :  StartArg EndArg                   { Match(@1, @2); }
    |  StartArg Params1 EndArg           { Match(@1, @3); }
    |  StartArg Params1 error            { CallHdlr("unmatched parentheses", @3); }
    |  StartArg error EndArg             { $$ = $3;
                                             CallHdlr("error in params", @2); }
    ;

MeasureName
    : TableRef ColumnRef                { CreateNewMeasure(@1, @2); }
    ;

QueryMeasureName
    : TableRef ColumnRef
    ;

ColMeasureRef
    : TableRef ColumnRef
    | ColumnRef
    ;

ColumnRef
    : COLUMNNAME
    ;

TableRef
    : TABLENAME 
    | ESCAPEDTABLENAME
    | FUNCTION  /* Very special case when a table name is the same as function name, but it is not wrapped in quotes */
    ;

TableExpression
    : FunctionCall 
    | TableRef
    ;

MeasureExpression
	: Expression					{   SpecifyMeasureExpression(@1); }
	;

QueryMeasureExpression
    : Expression
    ;

ScalarExpression
    : ColMeasureRef
    | FunctionCall
    | ParenthesisExpression
    | NUMBER
    | STRING
    | UnaryExpression
    | BinaryExpression
    | KWTRUE
    | KWFALSE
    | KWDAY
    | KWMONTH
    | KWYEAR
    ;

Expression
    : TableExpression
    | ScalarExpression
    ;

UnaryExpression
    : UnaryOperator Expression
    ;

UnaryOperator
    : '!' | '-'| '+'
    ;

BinaryExpression
    : Expression BinaryOperator Expression
    ;

BinaryOperator
    : '+' | '-' | '*'| '/'| AMPAMP | BARBAR | '&'
    | GT | GTE | LT | LTE | EQ | NEQ
    ;

ParenthesisExpression
    :  '(' ')'                   { Match(@1, @2); }
    |  '(' Params1 ')'           { Match(@1, @3); }
    |  '(' Params1 error         { CallHdlr("unmatched parentheses", @3); }
    ;

FunctionCall
	: FunctionName FunctionArgs
	;

FunctionName
	: FUNCTION							{ StartFunction(@1, $1.str); }
    | KWCALCULATE						{ StartFunction(@1, $1.str); }
    | KWTRUE						    { StartFunction(@1, $1.str); }
    | KWFALSE						    { StartFunction(@1, $1.str); }
	;

FunctionArgs
	: StartArg
    | ParenthesisParameters
    ;


StartArg
    : '('
    {
        StartParameters(@1);
    }
    ;
    
EndArg
    : ')'
    {
        EndParameters(@1);
    }
    ;

%%


