﻿/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.

THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.VisualStudio.Shell.Interop;
using System.Diagnostics;

namespace Microsoft.DaxEditor
{
    internal class LogWindowWriter : TextWriter
    {
        private readonly IVsOutputWindowPane _pane;
        public LogWindowWriter(IVsOutputWindowPane pane)
        {
            Debug.Assert(pane != null);
            this._pane = pane;
        }

        public void Clear()
        {
            this._pane.Clear();
        }

        public void Active()
        {
            this._pane.Activate();
        }

        public override System.Text.Encoding Encoding
        {
            get { return System.Text.Encoding.Default; }
        }

        public override void Write(string value)
        {
            this._pane.OutputStringThreadSafe(value);
        }

        public override void WriteLine(string value)
        {
            this._pane.OutputStringThreadSafe(value);
            this.WriteLine();
        }

        public override void WriteLine()
        {
            this._pane.OutputStringThreadSafe(Environment.NewLine);
        }
    }

}
